// HC11/CS8900 ethernet IP stack
//
// $Id: arp.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"

#include "cs8900.h"
#include "ip.h"
#include "serial.h"
#include "bootp.h"
#include "arp.h"

//-----------------------------------------------------------------
// Variables
XDATA u_int8_t arpMAC[ETHER_ADDR_LEN];
XDATA u_int8_t arpIP[INADDR_LEN];
DATA u_int8_t arpTimeout;
u_int8_t arpValid;

//-----------------------------------------------------------------
void
arp_init()
{
    arpValid = 0;
}

//-----------------------------------------------------------------
// ARP packet generation
void
arp_send(unsigned short int aOperation, XDATA const u_int8_t* aDestMAC, XDATA const u_int8_t* aDestIP)
{
   XDATA u_int8_t buffer;
   XDATA struct ether_header* ethHdr;
   XDATA struct ether_arp* arpPkt;

   buffer = cs8900_alloc_txbuffer(BUFFER_INUSE_NOFREE);
   ethHdr = (XDATA struct ether_header*) &txBuff[buffer][0];
   arpPkt = (XDATA struct ether_arp*) ((u_int8_t*) ethHdr + sizeof(struct ether_header));

   // abort if no spare buffers
   if (buffer == NO_BUFFERS_LEFT)
   {
      serial_print_string("\r\nNo transmit buffers available\r\n");
      return;
   }

   // can't send ARP if our address is not set
   if (ourIpValid == 1)
   {
      serial_print_debug('a');

      // build MAC header
      ethHdr->ether_type = htons(ETHERTYPE_ARP);
   
      // build ARP packet header
      arpPkt->ea_hdr.ar_hrd = htons(ARPHRD_ETHER);
      arpPkt->ea_hdr.ar_pro = htons(ETHERTYPE_IP);
      arpPkt->ea_hdr.ar_hln = ETHER_ADDR_LEN;
      arpPkt->ea_hdr.ar_pln = INADDR_LEN;
      arpPkt->ea_hdr.ar_op = htons(aOperation);

      switch(aOperation)
      {
         case ARPOP_REPLY :
         {
            serial_print_debug('1');

            // build arp reply
	    memcpy(ethHdr->ether_dhost, aDestMAC, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_sha, ourMAC, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_tha, &aDestMAC[0], ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_spa, ourIP, INADDR_LEN);
            memcpy(arpPkt->arp_tpa, &aDestIP[0], INADDR_LEN);

	    // send request/reply
	    cs8900_send(buffer, sizeof(struct ether_header) + sizeof(struct ether_arp));
            break;
         }
         case ARPOP_REQUEST :
         {
	    XDATA u_int8_t arpRetryCount;

            serial_print_debug('2');

            // build arp request
	    memset(ethHdr->ether_dhost, 0xff, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_sha, ourMAC, ETHER_ADDR_LEN);
	    memset(arpPkt->arp_tha, 0xff, ETHER_ADDR_LEN);
            memcpy(arpPkt->arp_spa, ourIP, INADDR_LEN);
            memcpy(arpPkt->arp_tpa, aDestIP, INADDR_LEN);

            // update table
            arpValid = 0;
            memcpy(arpMAC, aDestMAC, ETHER_ADDR_LEN);
            memcpy(arpIP, aDestIP, INADDR_LEN);

	    arpTimeout = 0;
	    arpRetryCount = 0;
	    while(arpValid != 1)
	    {
	       if (arpTimeout == 0)
	       {
		  if (arpRetryCount++ > MAXARPATTEMPTS)
		  {
		     serial_print_string("Aborting TX - ARP failed\r\n");
		     break;
		  }
		  
		  // send request/reply
		  cs8900_send(buffer, sizeof(struct ether_header) + sizeof(struct ether_arp));
		  arpTimeout = ARPINTERVAL;
	       }

	       // check for and process received packets
	       cs8900_poll();
	    }
            break;
         }
	 default:
	    break;
      }
    
   }

   cs8900_free_txbuffer(buffer);
}

//-----------------------------------------------------------------
// ARP packet receive & action
void
arp_receive()
{
   XDATA const struct ether_arp* arpPkt;

   arpPkt = (XDATA struct ether_arp*) (rxBuffer + sizeof(struct ether_header));

   serial_print_debug('A');

   if ((ntohs(arpPkt->ea_hdr.ar_hrd) == ARPHRD_ETHER)
       && (ntohs(arpPkt->ea_hdr.ar_pro) == ETHERTYPE_IP))
   {

      switch(ntohs(arpPkt->ea_hdr.ar_op))
      {
	 case ARPOP_REQUEST:
	 {
	    serial_print_debug('1');
#ifdef ARPDEBUG
	    serial_print_crlf();
	    serial_print_string("Got ARP request for ");
	    serial_print_int(arpPkt->arp_tpa[0]);
	    serial_send('.');
	    serial_print_int(arpPkt->arp_tpa[1]);
	    serial_send('.');
	    serial_print_int(arpPkt->arp_tpa[2]);
	    serial_send('.');
	    serial_print_int(arpPkt->arp_tpa[3]);
	    serial_print_crlf();
#endif
	    // respond to requests for our ip address
	    if (!memcmp(ourIP, arpPkt->arp_tpa, INADDR_LEN))
	    {
	       arp_send(ARPOP_REPLY, arpPkt->arp_sha, arpPkt->arp_spa);
	    }
	    break;
	 }
	 case ARPOP_REPLY:
	 {
#ifdef ARPDEBUG
	    u_int16_t i = 0;

	    serial_print_crlf();
	    serial_print_string("Got ARP reply for ");
	    serial_print_int(arpPkt->arp_spa[0]);
	    serial_send('.');
	    serial_print_int(arpPkt->arp_spa[1]);
	    serial_send('.');
	    serial_print_int(arpPkt->arp_spa[2]);
	    serial_send('.');
	    serial_print_int(arpPkt->arp_spa[3]);
	    serial_print_string(" MAC address is ");
            for(i = 0; i < 5; i++)
            {
               serial_print_hex_byte((u_int16_t) arpPkt->arp_sha[i], 0);
               serial_send(':');
            }
            serial_print_hex_byte((u_int16_t) arpPkt->arp_sha[5], 0);
	    serial_print_crlf();

	    serial_print_int(arpIP[0]);
	    serial_send('.');
	    serial_print_int(arpIP[1]);
	    serial_send('.');
	    serial_print_int(arpIP[2]);
	    serial_send('.');
	    serial_print_int(arpIP[3]);
	    serial_print_crlf();
#endif

            // check this the correct response
            if (!memcmp(arpPkt->arp_spa, arpIP, INADDR_LEN))
            {
               // update table
               memcpy(arpMAC, arpPkt->arp_sha, ETHER_ADDR_LEN);
               arpValid = 1;

	       serial_print_debug('2');
            }
	    break;
	 }
      }
   }
}

