// HC11/CS8900 ethernet IP stack
//
// $Id: bootp.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"
#include "cs8900.h"
#include "bootpdefs.h"
#include "serial.h"
#include "udp.h"
#include "ip.h"

#define BOOTPINTERVAL 5

//-----------------------------------------------------------------
// Globals
DATA u_int8_t bootpTimeout;

//-----------------------------------------------------------------
void
bootp_init()
{
   bootpTimeout = 0;
}

//-----------------------------------------------------------------
// send a bootp packet
void
bootp_send()
{
   XDATA u_int8_t buffer;
   static XDATA struct bootp* bootpPkt;
   static XDATA const u_int8_t destAddr[INADDR_LEN] = {0xff, 0xff, 0xff, 0xff};
   static XDATA const u_int8_t rfc1048cookie[INADDR_LEN] =  VM_RFC1048;

   buffer = cs8900_alloc_txbuffer(BUFFER_INUSE);

   // abort if no spare buffers
   if (buffer == NO_BUFFERS_LEFT)
   {
      serial_print_string("\r\nNo transmit buffers available\r\n");
      return;
   }

   bootpPkt = (XDATA struct bootp*) (&txBuff[buffer][0] + sizeof(struct ether_header) + sizeof(struct ip) + sizeof(struct udphdr));

   serial_print_debug('b');

   // fill packet inc. our hardware address
   bzero((XDATA void *) bootpPkt, sizeof(struct bootp));
   bootpPkt->bp_op = BOOTREQUEST;
   bootpPkt->bp_htype = HTYPE_ETHERNET;
   bootpPkt->bp_hlen = ETHER_ADDR_LEN;
   bootpPkt->bp_flags = htons(0x8000); // broadcast
   memcpy(bootpPkt->bp_vend, rfc1048cookie, INADDR_LEN);
   memcpy((void *) bootpPkt->bp_chaddr, (void *) ourMAC, ETHER_ADDR_LEN);

   udp_send(buffer, IPPORT_BOOTPC, IPPORT_BOOTPS, destAddr, sizeof(struct bootp));
   bootpTimeout = BOOTPINTERVAL;
}

//-----------------------------------------------------------------
// BOOTP packet receive & action
void
bootp_receive()
{
   static XDATA struct bootp* bootpPkt;

   bootpPkt = (XDATA struct bootp*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip) + sizeof(struct udphdr));

   serial_print_debug('B');

   // only interested in responses
   if (bootpPkt->bp_op == BOOTREPLY && 
       bootpPkt->bp_htype == HTYPE_ETHERNET &&
       ourIpValid == 0 &&
       !memcmp((void *) bootpPkt->bp_chaddr, (void *) ourMAC, ETHER_ADDR_LEN))
   {
      const u_int8_t rfc1048cookie[INADDR_LEN] =  VM_RFC1048;

      // get our IP address
      memcpy(ourIP, &bootpPkt->bp_yiaddr, INADDR_LEN);

      serial_print_string("\r\nGot bootp response - addr=");
      serial_print_int(bootpPkt->bp_yiaddr[0]);
      serial_send('.');
      serial_print_int(bootpPkt->bp_yiaddr[1]);
      serial_send('.');
      serial_print_int(bootpPkt->bp_yiaddr[2]);
      serial_send('.');
      serial_print_int(bootpPkt->bp_yiaddr[3]);
      
      // check for options
      if (!memcmp(bootpPkt->bp_vend, rfc1048cookie, INADDR_LEN))
      {
	 XDATA u_int8_t* mem;

	 mem = &bootpPkt->bp_vend[0] + INADDR_LEN;

	 while(*mem !=  TAG_END)
	 {
	    switch(*mem)
	    {
	       case TAG_GATEWAY:
		  break;
	       case TAG_IP_BROADCAST:
		  if (*(mem + 1) == 4)
		  {
		     // get broadcast address from vendor extension
		     memcpy(ourBCAST, mem + 2, INADDR_LEN);

		     serial_print_string(", broadcast=");
		     serial_print_int(*(mem + 2));
		     serial_send('.');
		     serial_print_int(*(mem + 3));
		     serial_send('.');
		     serial_print_int(*(mem + 4));
		     serial_send('.');
		     serial_print_int(*(mem + 5));
		  }
		  break;
	       default:
		  // ignore for now
		  break;
	    }

	    // skip TAG + length fields + option length
	    mem += *(mem + 1) + 2;
	 }
      }
      serial_print_crlf();

      // release flag
      ourIpValid = 1;
   }
}
