// DEV8051/CS8900 ethernet IP stack
//
// $Id: c8051.c,v 3.0 2002/04/10 11:58:28 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
// Serial code Copyright 1999, 2000 Free Software Foundation, Inc.
//         Written by Stephane Carrez (stcarrez@worldnet.fr)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"
#include "cs8900reg.h"
#include "i8051.h"
#include "cs8900.h"
#include "c8051F000.h"
#include "serial.h"

#define CS8900RESET P3_4
#define CS8900IOR   P3_5
#define CS8900IOW   P3_6

#define CS8900IDLE  0x60

//-----------------------------------------------------------------
// DEV8051 initialisation
void
cpu_init()
{
   unsigned char delay;

   // Enable external crystal	
   OSCXCN = 0x66;

   // disable watchdog timer
   WDTCN = 0xDE;
   WDTCN = 0xAD;

   // Delay >1 ms  before polling XTLVLD.
   delay=256;
   while(delay--);

   // Wait until external crystal has started
   while (!(OSCXCN & 0x80));

   // Switch to external oscillator
   OSCICN = 0x0C;

   // Disable internal oscillator; enable external oscillator
   OSCICN = 0x88;

   // Wait until external crystal has started
   while (!(OSCXCN & 0x80));
   
   // Switch to external oscillator
   OSCICN = 0x08;

   // Enable crossbar and weak pull-ups
   XBR2 = 0x40;

   // HW_UART routed to pins P0.0 and P0.1;
   XBR0 = 0x04;										

   // all pull-ups on
   PRT0CF = 0x01;   // P0.0 (HW TX)
   PRT1CF = 0xff;
   PRT3CF = 0xff;

   // reset CS8900
   CS8900IOR = 1;
   CS8900IOW = 1;
   CS8900RESET = 1;
   CS8900RESET = 0;

   // common 8051 initialisation
   i8051_init();
}

//-----------------------------------------------------------------
// read packet page register from cs8900
u_int16_t
cpu_cs8900_pp_reg_read(u_int16_t reg)
{
   u_int16_t result;

   // register select LSB
   P3 = CS8900_PP_ADDR_LSB | CS8900IDLE;
   P1 = (u_int8_t) (reg & 0xff);
   CS8900IOW = 0;
   CS8900IOW = 1;

   // register select MSB
   P3 = CS8900_PP_ADDR_MSB | CS8900IDLE;
   P1 = (u_int8_t) ((reg >> 8) & 0xff);
   CS8900IOW = 0;
   CS8900IOW = 1;

   // read LSB
   P3 = CS8900_PPDATA_PORT0_LSB | CS8900IDLE;
   CS8900IOR = 0;
   P1 = 0xff;
   result = P1;
   CS8900IOR = 1;

   // read MSB
   P3 = CS8900_PPDATA_PORT0_MSB | CS8900IDLE;
   CS8900IOR = 0;
   P1 = 0xff;
   result |= ((u_int16_t) P1) << 8;
   CS8900IOR = 1;

   return result;
}

//-----------------------------------------------------------------
// write to packet page register in cs8900
void
cpu_cs8900_pp_reg_write(u_int16_t reg, u_int16_t regdata)
{
   // register select LSB
   P3 = CS8900_PP_ADDR_LSB | CS8900IDLE;
   P1 = (u_int8_t) (reg & 0xff);
   CS8900IOW = 0;
   CS8900IOW = 1;

   // register select MSB
   P3 = CS8900_PP_ADDR_MSB | CS8900IDLE;
   P1 = (u_int8_t) ((reg >> 8) & 0xff);
   CS8900IOW = 0;
   CS8900IOW = 1;

   // data LSB
   P3 = CS8900_PPDATA_PORT0_LSB | CS8900IDLE;
   P1 = (u_int8_t) (regdata & 0xff);
   CS8900IOW = 0;
   CS8900IOW = 1;

   // data MSB
   P3 = CS8900_PPDATA_PORT0_MSB | CS8900IDLE;
   P1 = (u_int8_t) ((regdata >> 8) & 0xff);
   CS8900IOW = 0;
   CS8900IOW = 1;
}

//-----------------------------------------------------------------
u_int8_t 
cpu_cs8900_reg_read(u_int8_t reg)
{
   unsigned char result;

   P3 = reg | CS8900IDLE;
   CS8900IOR = 0;
   P1 = 0xff;
   result = P1;
   CS8900IOR = 1;

   return result;
}

//-----------------------------------------------------------------
void 
cpu_cs8900_reg_write(u_int8_t reg, u_int8_t regdata)
{
   P3 = reg | CS8900IDLE;
   P1 = regdata;
   CS8900IOW = 0;
   CS8900IOW = 1;
}
