// HC11/CS8900 ethernet IP stack
//
// $Id: cs8900.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#define MACTEMPLATE    "Tom(0)"

#include "ether.h"
#include "cs8900.h"
#include "cs8900reg.h"
#include "serial.h"
#include "arp.h"
#include "ip.h"

//-----------------------------------------------------------------
// Globals
XDATA u_int8_t txBuff[TXBUFFERCOUNT][BUFFERLEN];
XDATA u_int8_t rxBuffer[BUFFERLEN];
XDATA u_int8_t ourMAC[ETHER_ADDR_LEN];
static buffState_t txBuffState[TXBUFFERCOUNT];


//-----------------------------------------------------------------
// cs8900 library routines
//
static u_int8_t
cs8900_check()
{
   XDATA u_int16_t temp;

   temp = 0;

   temp = cpu_cs8900_pp_reg_read(PP_ChipID);

   if (temp == 0x630e)
   {
      serial_print_string("Found CS8900 - manufacturer ID 0x");
      serial_print_hex_int(temp, 1);
      serial_print_crlf();

      temp = cpu_cs8900_pp_reg_read(PP_ChipID + 2);

      serial_print_string("             - product ID 0x");
      serial_print_hex_int(temp, 1);
      serial_print_crlf();

      return 1;
   }
   
   return 0;
}

//-----------------------------------------------------------------
void
cs8900_init(u_int8_t aNodeId)
{
   XDATA u_int8_t i;
   
   i = 0;

   while (!cs8900_check())
   {
      serial_print_string("Failed to find CS8900\n\r");
   }

   // MAC address
   memcpy(ourMAC, MACTEMPLATE, ETHER_ADDR_LEN);
   if (aNodeId > 9)
   {
      ourMAC[4] = aNodeId + 'a' - 10;
   }
   else
   {
      ourMAC[4] = aNodeId + '0';
   }

   serial_print_string("             - set MAC Address to ");
   for(i = 0; i < 5; i++)
   {
      serial_print_hex_byte((u_int16_t) ourMAC[i], 0);
      serial_send(':');
   }
   serial_print_hex_byte((u_int16_t) ourMAC[5], 0);
   serial_print_crlf();

   while(!cpu_cs8900_pp_reg_read(PP_SelfST) & INIT_DONE)
   {
      serial_send('.');
   }

   // enable full duplex
   cpu_cs8900_pp_reg_write(PP_TestCTL, TESTCTL_FDX);

   // accept addressed and broadcast valid frames 
   cpu_cs8900_pp_reg_write(PP_RxCTL, RX_OK_ACCEPT     |
		       RX_IA_ACCEPT               |
		       RX_BROADCAST_ACCEPT);

   // set our MAC address
   cpu_cs8900_pp_reg_write(PP_IA,   (u_int16_t)ourMAC[1] << 8 | ourMAC[0]);
   cpu_cs8900_pp_reg_write(PP_IA+2, (u_int16_t)ourMAC[3] << 8 | ourMAC[2]);
   cpu_cs8900_pp_reg_write(PP_IA+4, (u_int16_t)ourMAC[5] << 8 | ourMAC[4]);

   // enable receiver and transmitter, force interface to 10baseT
   cpu_cs8900_pp_reg_write(PP_LineCTL, SERIAL_RX_ON | SERIAL_TX_ON);

   // mark all buffers as free
   for(i = 0; i < TXBUFFERCOUNT; i++)
   {
      txBuffState[i] = BUFFER_AVAILABLE;
   }
}

//-----------------------------------------------------------------
void
cs8900_send(u_int8_t aBuffer, u_int16_t aLength)
{
   register u_int16_t i;
   XDATA u_int8_t* txPtr;

   i = 0;
   txPtr = txBuff[aBuffer];

   serial_print_debug('s');

   // insert our MAC address
   memcpy((void *) (((struct ether_header *)txPtr)->ether_shost), (void *) ourMAC, ETHER_ADDR_LEN);
   
   // Initialise tx command register
   // start transmission only when all bytes are available
   cpu_cs8900_reg_write(CS8900_TxCMD_MSB, 0);
   cpu_cs8900_reg_write(CS8900_TxCMD_LSB, 0xc0);

#ifdef TXDEBUG
   serial_print_crlf();
   serial_print_string("Sending length = 0x");
   serial_print_hex_int(aLength, 0);
   serial_print_crlf();
#endif

   // write packet length
   cpu_cs8900_reg_write(CS8900_TxLength_MSB, (aLength >> 8) & 0xff);
   cpu_cs8900_reg_write(CS8900_TxLength_LSB, aLength & 0xff);

   // wait for packet space : TODO - TIMEOUT
   while ((cpu_cs8900_pp_reg_read(PP_BusST) & READY_FOR_TX_NOW) == 0) {};

   // pad odd packets
   if (aLength & 1)
   {
      *((XDATA u_int8_t *) txPtr + aLength) = 0;
   }

   // write to xmit buffer (convert length to words & round odd bytes up)
   aLength = (aLength + 1) >> 1;
   for (i = 1; i <= aLength; i++)
   {
#ifdef TXDEBUG
      serial_print_hex_byte(*txPtr, 1);
      serial_send(' ');
#endif

       cpu_cs8900_reg_write(CS8900_DATA_PORT0_LSB, *txPtr++);

#ifdef TXDEBUG
      serial_print_hex_byte(*txPtr, 1);
      serial_send(' ');
      if (i % 8 == 0) serial_print_crlf();
#endif

       cpu_cs8900_reg_write(CS8900_DATA_PORT0_MSB, *txPtr++);
   }
   
#ifdef TXDEBUG
   serial_print_crlf();
#endif

   if (txBuffState[aBuffer] == BUFFER_INUSE)
   {
      cs8900_free_txbuffer(aBuffer);
   }
}

//-----------------------------------------------------------------
void
cs8900_receive()
{
   XDATA u_int16_t rxStatus;
   XDATA u_int16_t rxLength;
   XDATA u_int8_t store;
   register u_int16_t i;
   register u_int8_t thisbyte;
   XDATA struct ether_header* ethHdr;

   rxStatus = 0;
   rxLength = 0;
   i = 0;
   store = 1;
   thisbyte = 0;
   ethHdr = (XDATA struct ether_header *) rxBuffer;

   serial_print_debug('R');

   // read length and status
   rxStatus = ((u_int16_t)cpu_cs8900_reg_read(CS8900_DATA_PORT0_MSB) << 8) | cpu_cs8900_reg_read(CS8900_DATA_PORT0_LSB);
   rxLength = ((u_int16_t)cpu_cs8900_reg_read(CS8900_DATA_PORT0_MSB) << 8) | cpu_cs8900_reg_read(CS8900_DATA_PORT0_LSB);

#ifdef RXDEBUG
   serial_print_crlf();
   serial_print_string("Receive length = 0x");
   serial_print_hex_int(rxLength, 0);
   serial_print_crlf();
#endif
   // discard long packets
   if (rxLength > BUFFERLEN) store = 0;

   // read packet in words
   for(i = 0; i < rxLength;i += 2)
   {
#ifdef RXDEBUG
      if (i % 16 == 0 && i != 0) serial_print_crlf();
#endif

      thisbyte = cpu_cs8900_reg_read(CS8900_DATA_PORT0_LSB);
      if (store)
      {
	 rxBuffer[i] = thisbyte;
      }

#ifdef RXDEBUG
      serial_print_hex_byte(thisbyte, 1);
      serial_send(' ');
#endif
      thisbyte = cpu_cs8900_reg_read(CS8900_DATA_PORT0_MSB);
      if (store)
      {
	 rxBuffer[i+1]   = thisbyte;
      }
#ifdef RXDEBUG
      serial_print_hex_byte(thisbyte, 1);
      serial_send(' ');
#endif
   }
#ifdef RXDEBUG
   serial_print_crlf();
#endif

   // analyse packet
   switch(ntohs(ethHdr->ether_type))
   {
      case ETHERTYPE_ARP:
      {
	 serial_print_debug('1');
	 arp_receive();
	 break;
      }
      case ETHERTYPE_IP:
      {
	 serial_print_debug('2');
         ip_receive();
	 break;
      }
   }
#ifdef RXDEBUG
   serial_print_crlf();
#endif
#ifdef DEBUG
   serial_print_crlf();
#endif
}

//-----------------------------------------------------------------
// poll the cs8900 for received packets
void 
cs8900_poll()
{
   XDATA u_int16_t rxEvent;

   rxEvent = cpu_cs8900_pp_reg_read(PP_RxEvent);
   
   if (rxEvent & RX_OK)
   {
      cs8900_receive();
   }
}

//-----------------------------------------------------------------
// dump a buffer
#ifdef DEBUG
void
cs8900_dump(XDATA u_int8_t* aBuffer, u_int16_t aLength)
{
   XDATA u_int16_t i;

   i = 0;

   serial_print_crlf();
	       
   for (i = 0; i < aLength;)
   {
      serial_print_hex_byte(aBuffer[i], 1);
      serial_send(' ');
      if (++i % 16 == 0) serial_print_crlf();
   }

   serial_print_crlf();
}
#endif

//-----------------------------------------------------------------
// Allocate a buffer - new state can be either BUFFER_INUSE_NOFREE or
// BUFFER_INUSE

u_int8_t
cs8900_alloc_txbuffer(buffState_t newState)
{
   static u_int8_t i = 0;

   for(i=0; i < TXBUFFERCOUNT; i++)
   {
      if (txBuffState[i] == BUFFER_AVAILABLE)
      {
	 txBuffState[i] = newState;
#ifdef MEMDEBUG
	 serial_print_string("\r\nAllocated buffer ");
	 serial_print_int(i);
	 serial_print_crlf();
#endif
	 
	 return i;
      }
   }
   
   return NO_BUFFERS_LEFT;
}

//-----------------------------------------------------------------
// free a transmit buffer

void
cs8900_free_txbuffer(u_int8_t aBuffer)
{
#ifdef MEMDEBUG
	 serial_print_string("\r\nFreed buffer ");
	 serial_print_int(aBuffer);
	 serial_print_string(" state was ");
	 serial_print_int(txBuffState[aBuffer]);
	 serial_print_crlf();
#endif
	 txBuffState[aBuffer] = BUFFER_AVAILABLE;
}
