// DEV8051/CS8900 ethernet IP stack
//
// $Id: dev8051.c,v 3.0 2002/04/10 11:59:05 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
// Serial code Copyright 1999, 2000 Free Software Foundation, Inc.
//         Written by Stephane Carrez (stcarrez@worldnet.fr)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"
#include "cs8900reg.h"
#include "i8051.h"
#include "cs8900.h"

#define CHIPEN P3_2

#define CS8900_WRITE_CYCLE 0x10


// Assembly function from cs8900io.asx
extern void select_reg(u_int8_t reg) _naked;


//-----------------------------------------------------------------
// DEV8051 initialisation
void
cpu_init()
{
   // common 8051 initialisation
   i8051_init();

   // de-activate chip select
   CHIPEN = 0;
}

//-----------------------------------------------------------------
// read packet page register from cs8900
u_int16_t
cpu_cs8900_pp_reg_read(u_int16_t reg)
{
   u_int16_t result;

   select_reg(CS8900_PP_ADDR_LSB | CS8900_WRITE_CYCLE);
   P1 = (u_int8_t) (reg & 0xff);
   CHIPEN = 1;
   CHIPEN = 0;

   select_reg(CS8900_PP_ADDR_MSB | CS8900_WRITE_CYCLE);
   P1 = (u_int8_t) ((reg >> 8) & 0xff);
   CHIPEN = 1;
   CHIPEN = 0;

   select_reg(CS8900_PPDATA_PORT0_LSB);
   P1 = 0xff;
   CHIPEN = 1;
   result = P1;
   CHIPEN = 0;

   select_reg(CS8900_PPDATA_PORT0_MSB);
   P1 = 0xff;
   CHIPEN = 1;
   result |= ((u_int16_t) P1) << 8;
   CHIPEN = 0;

   return result;
}

//-----------------------------------------------------------------
// write to packet page register in cs8900
void
cpu_cs8900_pp_reg_write(u_int16_t reg, u_int16_t regdata)
{
   select_reg(CS8900_PP_ADDR_LSB | CS8900_WRITE_CYCLE);
   P1 = (u_int8_t) (reg & 0xff);
   CHIPEN = 1;
   CHIPEN = 0;

   select_reg(CS8900_PP_ADDR_MSB | CS8900_WRITE_CYCLE);
   P1 = (u_int8_t) ((reg >> 8) & 0xff);
   CHIPEN = 1;
   CHIPEN = 0;

   select_reg(CS8900_PPDATA_PORT0_LSB | CS8900_WRITE_CYCLE);
   P1 = (u_int8_t) (regdata & 0xff);
   CHIPEN = 1;
   CHIPEN = 0;

   select_reg(CS8900_PPDATA_PORT0_MSB | CS8900_WRITE_CYCLE);
   P1 = (u_int8_t) ((regdata >> 8) & 0xff);
   CHIPEN = 1;
   CHIPEN = 0;
}

//-----------------------------------------------------------------
u_int8_t 
cpu_cs8900_reg_read(u_int8_t reg)
{
   u_int8_t result;

   select_reg(reg);
   P1 = 0xff;
   CHIPEN = 1;
   result = P1;
   CHIPEN = 0;

   return result;
}

//-----------------------------------------------------------------
void 
cpu_cs8900_reg_write(u_int8_t reg, u_int8_t regdata)
{
   select_reg(reg | CS8900_WRITE_CYCLE);
   P1 = regdata;
   CHIPEN = 1;
   CHIPEN = 0;
}
