// HC11/CS8900 ethernet IP stack
//
// $Id: ether.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"

#include "cs8900.h"
#include "cpu.h"
#include "udp.h"
#include "tcp.h"
#include "ip.h"
#include "arp.h"
#include "serial.h"
#include "timer.h"
#include "bootp.h"


#ifdef SDCC_mcs51
void time_of_day(void) interrupt 1 _naked;
#endif

//-----------------------------------------------------------------
int
main()
{
   u_int8_t nodeId;

   timer_init();
   cpu_init();
   nodeId = cpu_get_nodeid();

   serial_print_string("\r\nCS8900 IP (node = ");
   serial_print_hex_byte(nodeId, 0);
   serial_print_string(") $Revision: 3.0 $ - ");
   serial_print_string(__DATE__ " " __TIME__ "\r\n\r\n");

   cs8900_init(nodeId);
   ip_init(nodeId);

#ifdef TCP
   tcp_init();
#endif

#ifdef BOOTP
   bootp_init();
#endif

   // wait for rx event
   while (1)
   {
#ifdef BOOTP
      if (!ourIpValid && bootpTimeout == 0)
      {
	 // send bootp packet
	 bootp_send();
	 serial_send('.');
      }
#endif

#if 0
      // This is an example of a board initiated UDP transmit that
      // will generate ARP requests etc.
      if (ourIpValid)
      {
         const u_int8_t destIP[INADDR_LEN] = {203, 36, 78, 1};
	 u_int8_t buffer = cs8900_alloc_txbuffer(BUFFER_INUSE);
	 // abort if no spare buffers
	 if (buffer != NO_BUFFERS_LEFT)
	 {
	    udp_send(buffer, IPPORT_DISCARD, IPPORT_DISCARD, destIP, 0);
	 }
      }
#endif
      
      // check for received packet
      cs8900_poll();
   }

   return(0);
}


