// DEV8051/CS8900 ethernet IP stack
//
// $Id: i8051.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
// Serial code Copyright 1999, 2000 Free Software Foundation, Inc.
//         Written by Stephane Carrez (stcarrez@worldnet.fr)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"
#include "timer.h"
#include "serial.h"

/* 57600 baud @ 11.0592 MHz */
#define	BAUD_CONST 255

/* 15 interrupts per seconds @ 11.0592 MHz */
#define TIMER_TH0 0x10

data u_int8_t intcounter;

//-----------------------------------------------------------------
// common 8051 initialisation
void
i8051_init()
{
   // disable all interrupts
   EA = 0;

   // no serial interrupts
   ES = 0;

   // no timer interrupts
   ET1 = 0;
   ET0 = 0;
       
   // timer 1 for serial port
   TR1 = 0;
   TH1 = BAUD_CONST;
   TL1 = BAUD_CONST;
   TMOD = T1_M1;
   PCON = SMOD;
   TI = 1;
   SM1 = 1;
   SM0 = 1;
   REN = 1;

   // start timer 1
   TR1 = 1;

   // timer 0 init & start
   TR0 = 0;
   ET0 = 1;
   TH0 = TIMER_TH0;
   TL0 = 0;
   TMOD &= T1_MASK;
   TMOD |= T0_M0;
   TR0 = 1;

   // enable interrupts
   intcounter = 0;
   EA = 1;
}

//-----------------------------------------------------------------
// Get node ID from LSB of port E
u_int8_t
cpu_get_nodeid()
{
   // TODO: designation generation of node id
   return 1;
}

//-----------------------------------------------------------------
// Send a character out the serial port
void
cpu_serial_send(char c)
{
   while(!TI);
   TI = 0;
   SBUF = c;
}

//-----------------------------------------------------------------
void 
bzero(XDATA u_int8_t* s, u_int16_t n)
{
   register i;

   for (i=0; i<n; i++)
   {
      *(s++) = 0;
   }
}
   
//-----------------------------------------------------------------
u_int32_t htonl(u_int32_t aHostlong)
{
    // TODO
    return aHostlong;
}

//-----------------------------------------------------------------
u_int16_t htons(u_int16_t hostshort) _naked
{
   hostshort;
   _asm
      mov  a, dpl
      mov  b, dph
      mov  dpl, b
      mov  dph, a
      ret
   _endasm;
}

//-----------------------------------------------------------------
u_int32_t ntohl(u_int32_t aNetlong)
{
    // TODO
    return aNetlong;
}

//-----------------------------------------------------------------
u_int16_t ntohs(u_int16_t netshort) _naked
{
   netshort;
   _asm
      mov  a, dpl
      mov  b, dph
      mov  dpl, b
      mov  dph, a
      ret
   _endasm;
}

//-----------------------------------------------------------------
// evaluate 16 bit one's complement checksum over buffer
// aBuffer points to ip packet, aLength is length of ip payload
u_int16_t 
cpu_checksum(XDATA u_int16_t* aBuffer, u_int16_t aLength) _naked
{
   aBuffer;
   aLength;
_asm
   push       ar4
   push       ar5
   push       dpl
   push       dph
   mov        dptr, #_cpu_checksum_PARM_2 ; get aLength
   movx       a, @dptr
   mov        r5, a
   inc        dptr
   movx       a, @dptr
   mov        r4, a
   pop        dph
   pop        dpl			 
   push       ar2
   push       ar1					
   mov        r2, #0      ;r2/r1 hold sum
   mov        r1, #0
   mov        a, r4
   jnb        acc.7, acsum ;check non payload sum flag
   anl        a, #0x7f
   mov        r4, a       ;clear payload flag from length
   mov        r2, ar4     ;start sum with length
   mov        r1, ar5
   mov        a, dpl     ;point to protocol field in IP header
   add        a, #9
   mov        dpl, a
   clr        a
   addc       a, dph
   mov        dph, a
   movx       a, @dptr
   add        a, r1      ;add 8 bit protocol id to sum
   mov        r1, a
   clr        a
   addc       a, r2
   mov        r2, a
   inc        dptr       ;move pointer to src address
   inc        dptr
   inc        dptr
   mov        a, r5      ;8 more bytes to main loop
   add        a, #8
   mov        r5, a
   clr        a
   addc       a, r4
   mov        r4,a
acsum:
   clr        c           ;convert length into words (divide by 2)
   mov        a, r4
   rrc        a
   mov        r4, a
   mov        a, r5
   rrc        a
   mov        r5, a
   mov        b, #0			      
   jnc        aceven
   inc        b            ;flag odd length packet
aceven:
   inc        r4           ;extra count msb to allow for djnz
   clr        c
acloop:
   movx       a, @dptr     ;add msb
   addc       a, r2
   mov        r2, a
   inc        dptr
   movx       a, @dptr     ;add lsb
   addc       a, r1
   mov        r1, a
   inc        dptr
   djnz       r5, acloop
   djnz       r4, acloop
   mov        a, b
   jz         aciseven     ;check for odd length packet
   movx       a, @dptr     ;add msb
   addc       a, r2
   mov        r2, a
   clr        a
   addc       a, r1
   mov        r1, a
   mov        a, r2
   sjmp       acdone
aciseven:
   clr        a		   ;collect last carry flag
   addc       a, r2
acdone:
   cpl        a
   mov        dpl, a
   mov        a, r1        ;complement result
   cpl        a
   mov        dph, a
   pop        ar1
   pop        ar2
   pop        ar5
   pop        ar4
   ret
_endasm;			 
	 
}

//-----------------------------------------------------------------
// Time of day ISR and maintenance

void
time_of_day(void) interrupt 1 _naked
{
_asm
        push    acc
        inc     _intcounter
        mov     a, _intcounter
        cjne    a, #15, 000001$
        mov     _intcounter, #0
        lcall   _timer_per_second
#ifdef SERIAL
#ifdef DEBUG
	jnb	ti, .
	clr	ti		;clr ti before the mov to sbuf!
	mov	sbuf, #0x21
#endif
#endif
000001$:
        mov	th0, #TIMER_TH0
	mov	tl0, #0
	setb	tr0
        pop     acc				  
	reti
_endasm;
}

