// HC11/CS8900 ethernet IP stack
//
// $Id: icmp.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"

#include "ip.h"
#include "serial.h"
#include "cs8900.h"

#define UNREACHABLE_PAYLOAD_LEN (8 + sizeof(struct ip))

//-----------------------------------------------------------------
// send an ICMP echo response using the received request data
void
icmp_send(u_int8_t aType, u_int8_t aCode)
{
   XDATA u_int8_t buffer;
   XDATA struct ip* rxIpPkt;
   XDATA struct icmphdr* rxIcmpPkt;
   XDATA struct ip* ipPkt;
   XDATA struct icmphdr* icmpPkt;
   XDATA u_int16_t icmpPktLen;

   buffer = cs8900_alloc_txbuffer(BUFFER_INUSE);
   rxIpPkt = (XDATA struct ip *) (rxBuffer + sizeof(struct ether_header));
   rxIcmpPkt = (XDATA struct icmphdr *) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));
   ipPkt = (XDATA struct ip *) (&txBuff[buffer][0] + sizeof(struct ether_header));
   icmpPkt = (XDATA struct icmphdr *) ((u_int8_t*) ipPkt + sizeof(struct ip));
   icmpPktLen = 0;

   // abort if no spare buffers
   if (buffer == NO_BUFFERS_LEFT)
   {
      serial_print_string("\r\nNo transmit buffers available\r\n");
      return;
   }

   serial_print_debug('p');

   // datafill common icmp header components
   icmpPkt->type = aType;
   icmpPkt->subcode = aCode;

   switch(aType)
   {
      case ICMP_ECHOREPLY:
	 serial_print_debug('1');
	 icmpPkt->un.echo.id = rxIcmpPkt->un.echo.id;
	 icmpPkt->un.echo.sequence = rxIcmpPkt->un.echo.sequence;

	 // length is same as rx
	 ipPkt->ip_len = rxIpPkt->ip_len;
	 icmpPktLen = ntohs(rxIpPkt->ip_len) - sizeof(struct ip);

	 // copy payload
	 memcpy((u_int8_t*) icmpPkt + sizeof(struct icmphdr),
		(u_int8_t*) rxIcmpPkt + sizeof(struct icmphdr),
		icmpPktLen - sizeof(struct icmphdr));
	 break;

      case ICMP_DEST_UNREACH:
	 serial_print_debug('2');

	 // unused fields zero
	 icmpPkt->un.echo.id = 0;
	 icmpPkt->un.echo.sequence = 0;

	 // length is constant
	 icmpPktLen = sizeof(struct icmphdr) + UNREACHABLE_PAYLOAD_LEN;
	 ipPkt->ip_len = htons(icmpPktLen + sizeof(struct ip));

	 // copy IP header and 8 bytes of data from received
	 // packet. nb IP options are not supported, data is assumed
	 // to be immediately after static header
	 memcpy((u_int8_t*) icmpPkt + sizeof(struct icmphdr),
		(u_int8_t*) rxIpPkt,
		sizeof(struct ip) + UNREACHABLE_PAYLOAD_LEN);
	 break;
      default:
	 cs8900_free_txbuffer(buffer);
	 return;
	 break;
   }

   // replace destination IP with source of rx packet
   memcpy(&ipPkt->ip_dst, &rxIpPkt->ip_src, INADDR_LEN);
   
   // protocol, ttl & id
   ipPkt->ip_p = IPPROTO_ICMP;
   ipPkt->ip_id = rxIpPkt->ip_id;
   ipPkt->ip_ttl = rxIpPkt->ip_ttl;

   // checksum and send
   icmpPkt->checksum = 0;
   icmpPkt->checksum = cpu_checksum((XDATA u_int16_t *) icmpPkt, icmpPktLen);
   ip_send(buffer);
}

//-----------------------------------------------------------------
// ICMP packet receive & action
void
icmp_receive(u_int16_t aLength)
{
   XDATA struct icmphdr* icmpPkt;

   icmpPkt = (XDATA struct icmphdr*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));

   serial_print_debug('P');

   if (cpu_checksum((XDATA u_int16_t *) icmpPkt, aLength) == 0)
   {
      switch(icmpPkt->type)
      {
	 case ICMP_ECHO:
	    serial_print_debug('1');
	    icmp_send(ICMP_ECHOREPLY, 0);
	    break;
      }
   }
   else
   {
      serial_print_string("\r\nICMP checksum error\r\n");
#ifdef DEBUG
      cs8900_dump(rxBuffer, BUFFERLEN);
#endif
   }
}

