// HC11/CS8900 ethernet IP stack
//
// $Id: ip.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"
#include "udp.h"
#include "arp.h"
#include "icmp.h"
#include "tcp.h"
#include "bootp.h"

#include "serial.h"
#include "cs8900.h"


//-----------------------------------------------------------------
// Globals
XDATA u_int8_t ourIP[INADDR_LEN];
XDATA u_int8_t gateIP[INADDR_LEN];
XDATA u_int8_t ourBCAST[INADDR_LEN];
u_int8_t ourIpValid;

//-----------------------------------------------------------------
void
ip_init(u_int8_t aNodeId)
{
   // initial IP broadcast address
   memset(ourBCAST, 0xff, INADDR_LEN);
#ifdef BOOTP   
   aNodeId = aNodeId;

   ourIpValid = 0;

   serial_print_string("\r\nBootp .");
#else
   // not using bootp so fix our IP address
   ourIP[0] = IPADDRESS_0;
   ourIP[1] = IPADDRESS_1;
   ourIP[2] = IPADDRESS_2;
   ourIP[3] = IPADDRESS_3 + aNodeId;

   // class C (a.b.c.255)
   ourBCAST[0] = IPADDRESS_0;
   ourBCAST[1] = IPADDRESS_1;
   ourBCAST[2] = IPADDRESS_2;
   ourBCAST[3] = 255;

   // gateway
   gateIP[0] = IPADDRESS_0;
   gateIP[1] = IPADDRESS_1;
   gateIP[2] = IPADDRESS_2;
   gateIP[3] = GATEADDRESS_3;

   ourIpValid = 1;

   serial_print_string("             - fixed ip address is ");
   serial_print_int(ourIP[0]);
   serial_send('.');
   serial_print_int(ourIP[1]);
   serial_send('.');
   serial_print_int(ourIP[2]);
   serial_send('.');
   serial_print_int(ourIP[3]);
   serial_print_crlf();

   serial_print_string("             - broadcast address is ");
   serial_print_int(ourBCAST[0]);
   serial_send('.');
   serial_print_int(ourBCAST[1]);
   serial_send('.');
   serial_print_int(ourBCAST[2]);
   serial_send('.');
   serial_print_int(ourBCAST[3]);
   serial_print_crlf();

   serial_print_string("             - gateway address is ");
   serial_print_int(gateIP[0]);
   serial_send('.');
   serial_print_int(gateIP[1]);
   serial_send('.');
   serial_print_int(gateIP[2]);
   serial_send('.');
   serial_print_int(gateIP[3]);
   serial_print_crlf();
#endif
}

//-----------------------------------------------------------------
// send an ip packet (protocol, length & destination address are
// assumed to be filled out).  Packet is MAC size packet ready to go.
void
ip_send(u_int8_t aBuffer)
{
   XDATA struct ether_header* ethHdr;
   XDATA struct ip* ipPkt;

   ethHdr = (XDATA struct ether_header*) txBuff[aBuffer];
   ipPkt = (XDATA struct ip*) (((u_int8_t*) ethHdr) + sizeof(struct ether_header));

   serial_print_debug('i');
   ipPkt->ip_hl_v = IPVERSION_HEADERLEN;
   ipPkt->ip_tos = 0;
   ipPkt->ip_off = 0;

   // check bootp status
   if (ourIpValid == 1)
   {
      serial_print_debug('1');

      // our ip address is assigned so use our IP & a specific MAC address
      memcpy(&ipPkt->ip_src, ourIP, INADDR_LEN);

      if ((ipPkt->ip_dst[0] == ourIP[0]) &&
	  (ipPkt->ip_dst[1] == ourIP[1]) &&
	  (ipPkt->ip_dst[2] == ourIP[2]))
      {
	 // on this subnet so arp destination if unknown
	 if (memcmp(&ipPkt->ip_dst, arpIP, INADDR_LEN))
	 {
	    arp_send(ARPOP_REQUEST, 0, (XDATA u_int8_t *) &ipPkt->ip_dst);
	 }
      }
      else
      {
	 if (memcmp(gateIP, arpIP, INADDR_LEN))
	 {
	    // off this subnet arp gateway
	    arp_send(ARPOP_REQUEST, 0, gateIP);
	 }
      }

      if (arpValid == 0)
      {
	 // abort, failed ARP
	 return;
      }
      
      memcpy(ethHdr->ether_dhost, arpMAC, ETHER_ADDR_LEN);
   }
   else
   {
      // use null IP address & bcast MAC address if bootp is not finished

      serial_print_debug('2');
      memset(ethHdr->ether_dhost, 0xff, ETHER_ADDR_LEN);
      bzero((XDATA void *) &ipPkt->ip_src, INADDR_LEN); 
   }

   // checksum
   ipPkt->ip_sum = 0;
   ipPkt->ip_sum = cpu_checksum((XDATA u_int16_t *) ipPkt, sizeof(struct ip));

   // MAC
   ethHdr->ether_type = htons(ETHERTYPE_IP);

   cs8900_send(aBuffer, ntohs(ipPkt->ip_len) + sizeof(struct ether_header));
}

//-----------------------------------------------------------------
// IP packet receive & action
void
ip_receive()
{
   XDATA const struct ip* ipPkt;

   ipPkt = (XDATA struct ip*) (rxBuffer + sizeof(struct ether_header));

   serial_print_debug('I');

   // extract IPv4, non fragmented, optionless packets with a valid length
   // destined for us (if bootp is done) or broadcast
   if ((ipPkt->ip_hl_v == IPVERSION_HEADERLEN) &&
       ((ntohs(ipPkt->ip_off) & IP_OFFMASK) == 0) && 
       (ntohs(ipPkt->ip_len) > 20) &&
       ((memcmp(ourIP, &ipPkt->ip_dst, INADDR_LEN) == 0 && (ourIpValid == 1)) ||
	(memcmp(ourBCAST, &ipPkt->ip_dst, INADDR_LEN) == 0)))
   {
      // save MAC address and IP address to save ARP
      memcpy(arpMAC, ((struct ether_header *)rxBuffer)->ether_shost, ETHER_ADDR_LEN);
      memcpy(arpIP, &ipPkt->ip_src, INADDR_LEN);
      arpValid = 1;

      serial_print_debug('1');

      if (cpu_checksum((XDATA u_int16_t *) ipPkt, 20) == 0)
      {
	 serial_print_debug('2');

	 // Valid packet
	 switch(ipPkt->ip_p)
	 {
#ifdef ICMP
	    case IPPROTO_ICMP:
	    {
	       icmp_receive(ntohs(ipPkt->ip_len) - sizeof(struct ip));
	       break;
	    }
#endif
#ifdef UDP
	    case IPPROTO_UDP:
	    {
	       udp_receive(ntohs(ipPkt->ip_len) - sizeof(struct ip));
	       break;
	    }
#endif
#ifdef TCP
            case IPPROTO_TCP:
            {
               tcp_receive(ntohs(ipPkt->ip_len) - sizeof(struct ip));
	       break;
	    }
#endif
	    default:
               break;
	       
	 }
      }
      else
      {
	 serial_print_string("IP header checksum error\r\n");
      }
   }
   
}

