// HC11/CS8900 ethernet IP stack
//
// $Id: tcp.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#define TCPMAXCONN     10

#include "ether.h"

#include "ip.h"
#include "serial.h"
#include "cs8900.h"
#include "arp.h"
#include "tcpstruct.h"
#include "icmp.h"

#include "telnet.h"
#include "http.h"

//-----------------------------------------------------------------
// Variables
u_int32_t tcpStartSeq;
static XDATA tcpConn tcpTable[TCPMAXCONN];        

//-----------------------------------------------------------------
// dump contents of tcp packet
#ifdef DEBUG
void tcp_dump(const XDATA struct ip* packet)
{
   XDATA struct tcphdr* tcpPkt;
   XDATA u_int32_t* ipaddr;

   tcpPkt = (XDATA struct tcphdr*) ((XDATA u_int8_t *) packet + sizeof(struct ip));

   serial_print_string("\r\nTCP Packet dump\r\n  source ip = 0x");
   ipaddr = (XDATA u_int32_t *) &packet->ip_src[0];
   serial_print_hex_int(ntohl(*ipaddr) >> 16, 1);
   serial_print_hex_int(ntohl(*ipaddr) & 0xffff, 1);

   serial_print_string("\r\n  dest ip = 0x");
   ipaddr = (XDATA u_int32_t *) &packet->ip_dst[0];
   serial_print_hex_int(ntohl(*ipaddr) >> 16, 1);
   serial_print_hex_int(ntohl(*ipaddr) & 0xffff, 1);

   serial_print_string("\r\n  ip length (bytes) = ");
   serial_print_int(ntohs(packet->ip_len));

   serial_print_string("\r\n  source port = ");
   serial_print_int(ntohs(tcpPkt->source));

   serial_print_string("\r\n  dest port = ");
   serial_print_int(ntohs(tcpPkt->dest));

   serial_print_string("\r\n  seq = 0x");
   serial_print_hex_int(ntohl(tcpPkt->seq) >> 16, 1);
   serial_print_hex_int(ntohl(tcpPkt->seq) & 0xffff, 1);

   serial_print_string("\r\n  ack seq = 0x");
   serial_print_hex_int(ntohl(tcpPkt->ack_seq) >> 16, 1);
   serial_print_hex_int(ntohl(tcpPkt->ack_seq) & 0xffff, 1);

   serial_print_string("\r\n  tcp header length (bytes) = ");
   serial_print_int(tcpPkt->doff * sizeof(u_int32_t));
   
   serial_print_string("\r\n  flags = ");
   if (tcpPkt->fin) serial_send('F');
   if (tcpPkt->syn) serial_send('S');
   if (tcpPkt->rst) serial_send('R');
   if (tcpPkt->psh) serial_send('P');
   if (tcpPkt->ack) serial_send('A');
   if (tcpPkt->urg) serial_send('U');

   serial_print_string("\r\n  window = ");
   serial_print_int(ntohs(tcpPkt->window));

   serial_print_string("\r\n  check = 0x");
   serial_print_hex_int(ntohs(tcpPkt->check), 1);

   serial_print_string("\r\n  urgent pointer = 0x");
   serial_print_hex_int(ntohs(tcpPkt->urg_ptr), 1);

   serial_print_crlf();
}
#endif

//-----------------------------------------------------------------
// send a tcp packet given the connection, flags and payload length
void
tcp_send(XDATA tcpConn* aConn,
	 u_int8_t flags,
	 u_int16_t aLength)
{
   XDATA struct ip* ipPkt;
   XDATA struct tcphdr* tcpPkt;

   ipPkt = (XDATA struct ip*) (&txBuff[aConn->buffer][0] + sizeof(struct ether_header));
   tcpPkt = (XDATA struct tcphdr*) ((XDATA u_int8_t*) ipPkt + sizeof(struct ip));

   serial_print_debug('t');

   // TCP header
   tcpPkt->source = htons(aConn->ourPort);
   tcpPkt->dest = htons(aConn->farPort);
   tcpPkt->seq = htonl(aConn->ourSeq);
   if (flags & TCP_FLAG_ACK)
   {
      tcpPkt->ack_seq = htonl(aConn->farSeq + 1);
      tcpPkt->ack = 1;
   }
   else
   {
      tcpPkt->ack_seq = 0;
      tcpPkt->ack = 0;
   }
   tcpPkt->res1 = 0;
   tcpPkt->res2 = 0;
   tcpPkt->fin = (flags & TCP_FLAG_FIN)?1:0;
   tcpPkt->syn = (flags & TCP_FLAG_SYN)?1:0;
   tcpPkt->rst = (flags & TCP_FLAG_RST)?1:0;
   tcpPkt->psh = (flags & TCP_FLAG_PSH)?1:0;
   tcpPkt->urg = (flags & TCP_FLAG_URG)?1:0;
   tcpPkt->window = htons(BUFFERLEN 
			  - sizeof(struct ether_header) 
			  - sizeof(struct ip) 
			  - sizeof(struct tcphdr) 
			  - sizeof(u_int32_t));
   tcpPkt->check = 0;
   tcpPkt->urg_ptr = 0;

   // extra 4 bytes for MSS option
   aLength += sizeof(struct tcphdr) + sizeof(u_int32_t);
   tcpPkt->doff = (sizeof(struct tcphdr) + sizeof(u_int32_t))/ sizeof(u_int32_t); 
   ipPkt->ip_len = htons(aLength + sizeof(struct ip));

   // MSS option (max length is buffer length without MAC/IP/TCP + MSS)
   *((u_int8_t *) tcpPkt + sizeof(struct tcphdr)) = 0x2;
   *((u_int8_t *) tcpPkt + sizeof(struct tcphdr) + 1) = 0x4;  // 4 bytes
   *((u_int16_t *) ((u_int8_t *) tcpPkt + sizeof(struct tcphdr) + 2)) = 
      htons(BUFFERLEN 
	    - sizeof(struct ether_header) 
	    - sizeof(struct ip) 
	    - sizeof(struct tcphdr) 
	    - sizeof(u_int32_t));

   // IP header
   memcpy(&ipPkt->ip_dst, &aConn->farAddr, INADDR_LEN);
   memcpy(&ipPkt->ip_src, ourIP, INADDR_LEN);
   ipPkt->ip_p = IPPROTO_TCP;
   ipPkt->ip_id = 0;
   ipPkt->ip_ttl = 64;

   // insert checksum & send
   tcpPkt->check = cpu_checksum((XDATA u_int16_t *) ipPkt, (aLength | CPU_CHECKSUM_INC_IP));

   ip_send(aConn->buffer);
}

//-----------------------------------------------------------------
// Locate a connection entry in the table given params.  If no match is
// found, the first free connection is returned, if none is available
// then zero is returned.  All params are in network byte ordering.
static XDATA tcpConn*
tcp_find_conn(XDATA u_int8_t* farAddr, u_int16_t farPort, u_int16_t ourPort)
{
   register u_int8_t i;
   u_int8_t firstFree;
   const XDATA u_int32_t* ipAddr;
    
   i = 0;
   firstFree = TCPMAXCONN;
   ipAddr = (XDATA u_int32_t *) farAddr;
    
   for (i = 0; i < TCPMAXCONN; i++)
   {
      if (tcpTable[i].state == LISTEN)
      {
	 if (firstFree == TCPMAXCONN) firstFree = i;
      } 
      else if (tcpTable[i].farAddr == *ipAddr &&
	       tcpTable[i].farPort == farPort &&
	       tcpTable[i].ourPort == ourPort)
      {
	 return &tcpTable[i];
      }
   }
    
   if (firstFree == TCPMAXCONN)
   {
      return 0;
   }
   else
   {
      XDATA tcpConn* entry = &tcpTable[firstFree];
      entry->farAddr = *ipAddr;
      entry->farPort = farPort;
      entry->ourPort = ourPort;
      entry->buffer = INVALID_BUFFER;

      return entry;
   }
}

//-----------------------------------------------------------------
// TCP packet receive & action
void
tcp_receive(u_int16_t aLength)
{
   const XDATA struct tcphdr* tcpPkt;
   const XDATA struct ip* ipPkt;

   tcpPkt = (const XDATA struct tcphdr*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));
   ipPkt = (const XDATA struct ip*) (rxBuffer + sizeof(struct ether_header));

   serial_print_debug('T');

   if (cpu_checksum((XDATA u_int16_t *)(rxBuffer + sizeof(struct ether_header)),
		    (CPU_CHECKSUM_INC_IP | aLength)) == 0)
   {
      serial_print_string("\r\nTCP checksum error\r\n");
#ifdef DEBUG
      cs8900_dump(rxBuffer, BUFFERLEN);
#endif
   }
   else
   {
      XDATA tcpConn* conn;

      switch(tcpPkt->dest)
      {
	 case 0:
#ifdef TELNET
	 case 23: // Telnet
#endif
#ifdef HTTP
	 case 80: // HTTP
#endif
	    // do nothing for valid ports
	    break;
	 default:
	    serial_print_debug('?');
#ifdef ICMP	       
	    icmp_send(ICMP_DEST_UNREACH, ICMP_PORT_UNREACH);
#endif
	    return;
	    break;
      }
      
      conn = tcp_find_conn((XDATA u_int8_t *)ipPkt->ip_src, ntohs(tcpPkt->source), ntohs(tcpPkt->dest));

      if (conn == 0)
      {
	 serial_print_string("\r\nTCP maximum connections exceeded\r\n");
	 return;
      }

      // update for ack of our packet
      if (conn->buffer != INVALID_BUFFER &&
	  tcpPkt->ack &&
	  (ntohs(tcpPkt->ack_seq) == conn->ourSeq + 1))
      {
	 cs8900_free_txbuffer(conn->buffer);
	 conn->buffer = INVALID_BUFFER;
	 conn->ourSeq++;
      }

      if (conn->state != LISTEN && ntohs(tcpPkt->seq) != conn->farSeq + 1)
      {
	 // discard out of order packets
	 serial_print_debug('?');
	 return;
      }

      // current state
      switch(conn->state)
      {
	 case LISTEN:
	    if (tcpPkt->syn)
	    {
	       serial_print_debug('1');
	       conn->state = SYN_RCVD;
	       conn->ourSeq = ntohl(tcpStartSeq);
	       conn->farSeq = htonl(tcpPkt->seq);

	       // discard is retrying previous packet
	       if (conn->buffer != INVALID_BUFFER)
	       {
		  // discard packet
		  return;
	       }		  

	       // send syn + ack
	       conn->buffer = cs8900_alloc_txbuffer(BUFFER_INUSE_NOFREE);
	       tcp_send(conn, TCP_FLAG_ACK | TCP_FLAG_SYN, 0);
	    }
	    else
	    {
	       // discard packet
	       return;
	    }		  
	    break;
	 case SYN_RCVD:
	    if (tcpPkt->ack && conn->buffer == INVALID_BUFFER)
	    {
	       conn->state = ESTABLISHED;
	    } 
	    else
	    {
	       // discard packet
	       return;
	    }
	    break;
	 case ESTABLISHED:
	    if (conn->buffer != INVALID_BUFFER)
	    {
	       // can't ack so discard packet
	       return;
	    }
	    
	    // ack packet
	    conn->farSeq += 
	    conn->buffer = cs8900_alloc_txbuffer(BUFFER_INUSE_NOFREE);
	    tcp_send(conn, TCP_FLAG_ACK, 0);

	    if (tcpPkt->fin)
	    {
	       serial_print_debug('3');
	       conn->state = LAST_ACK;
	       tcp_send(conn, TCP_FLAG_FIN, 0);
	    }
	    break;
	 case LAST_ACK:
	    if (tcpPkt->ack)
	    {
	       // TODO : check seq being ack'd
	       serial_print_debug('4');
	       conn->state = LISTEN;
	    }
	    break;
      }
	 
      // if connection established and data present then pass to
      // handler
      if ((conn->state == ESTABLISHED) &&
	  (ipPkt->ip_len != (tcpPkt->doff * sizeof(u_int32_t)) + sizeof(struct ip)))
      {
	 switch(tcpPkt->dest)
	 {
#ifdef TELNET
	    case 23: // Telnet
	       telnet_receive(conn, aLength);
	       break;
#endif
#ifdef HTTP
	    case 80: // HTTP
	       http_receive(conn, aLength);
	       break;
#endif
	    default:
	       break;
	 }
      }
   }
}

//-----------------------------------------------------------------
// initialise the tcp state machine for all connections
void 
tcp_init()
{
   u_int8_t i = 0;

   for(i=0; i < TCPMAXCONN; i++)
   {
      tcpTable[i].farAddr = 0;
      tcpTable[i].state = LISTEN;
      tcpTable[i].ourPort = 0;
      tcpTable[i].farPort = 0;
      tcpTable[i].ourSeq = 0;
      tcpTable[i].farSeq = 0;
   }
   
   tcpStartSeq = 0;
}

