// HC11/CS8900 ethernet IP stack
//
// $Id: udp.c,v 3.0 2002/04/10 11:57:42 tom Exp tom $
//
// Copyright (C) 2000-1 Tom Parker (tom-hc11@wiresncode.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Author's note: This is free software, for commerical use I would
// appreciate some recognition though.  If you are using this and like
// it send me some email or a postcard.  Code changes for inclusion
// are also welcome.
//
// Tom Parker, PO Box 2165, Fitzroy, VIC, 3065, AUSTRALIA (tom-hc11@wiresncode.com)
//

#include "ether.h"

#include "ntp.h"
#include "bootp.h"
#include "bootpdefs.h"
#include "ip.h"
#include "serial.h"
#include "cs8900.h"
#include "timer.h"
#include "icmp.h"

//-----------------------------------------------------------------
// send a udp packet given the address, length & payload in txBuffer
void
udp_send(u_int8_t aBuffer,
         u_int16_t aSourcePort, 
	 u_int16_t aDestPort, 
	 XDATA const u_int8_t* aAddress, 
	 u_int16_t aLength)
{
   static XDATA struct ip* ipPkt;
   static XDATA struct udphdr* udpPkt;

   ipPkt = (XDATA struct ip*) (&txBuff[aBuffer][0] + sizeof(struct ether_header));
   udpPkt = (XDATA struct udphdr*) ((u_int8_t*) ipPkt + sizeof(struct ip));

   serial_print_debug('u');

   // UDP header
   udpPkt->source = htons(aSourcePort);
   udpPkt->dest = htons(aDestPort);
   aLength += sizeof(struct udphdr);
   udpPkt->len = htons(aLength);
   udpPkt->check = 0;

   // destination IP
   memcpy(&ipPkt->ip_dst, aAddress, INADDR_LEN);

   // fill source address for checksum - use a zero ip address if we
   // do not have one (eg sending bootp)
   if (ourIpValid == 1)
   {
      serial_print_debug('1');
      memcpy(&ipPkt->ip_src, ourIP, INADDR_LEN);
   }
   else
   {
      serial_print_debug('2');
      bzero((XDATA void *) &ipPkt->ip_src, INADDR_LEN);
   }

   // packet length, protocol & id are the same
   aLength += sizeof(struct ip);
   ipPkt->ip_len = htons(aLength);
   ipPkt->ip_p = IPPROTO_UDP;
   ipPkt->ip_id = 0;
   ipPkt->ip_ttl = 64;

   // insert checksum & send
   udpPkt->check = 0;
   udpPkt->check = cpu_checksum((XDATA u_int16_t *) ipPkt, (aLength | CPU_CHECKSUM_INC_IP) - sizeof(struct ip));
   ip_send(aBuffer);
}

#ifdef NTP
//-----------------------------------------------------------------
// NTP packet receive & action
void
udp_ntp_receive(u_int16_t aLength)
{
   serial_print_debug('N');

   if (aLength >= sizeof(struct ntp))
   {
      struct ntp* ntpPkt = (struct ntp*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip) + sizeof(struct udphdr));

      currentTime = ntpPkt->xmt.l_ui;
   }
   else
   {
      serial_print_string("\r\nUndersize NTP packet received ");
      serial_print_int(aLength);
      serial_print_string(" < ");
      serial_print_int(sizeof(struct ntp));
      serial_print_crlf();
   }
}
#endif
//-----------------------------------------------------------------
// UDP packet receive & action
void
udp_receive(u_int16_t aLength)
{
   static XDATA struct udphdr* udpPkt;

   udpPkt = (XDATA struct udphdr*) (rxBuffer + sizeof(struct ether_header) + sizeof(struct ip));

   serial_print_debug('U');

   // length check
   if (aLength == ntohs(udpPkt->len))
   {
      // checksum
      if (cpu_checksum((XDATA u_int16_t *)(rxBuffer + sizeof(struct ether_header)),
		       (CPU_CHECKSUM_INC_IP | aLength)) == 0)
      {
	 switch(ntohs(udpPkt->dest))
	 {
#ifdef NTP
	    case 123: // NTP
	       serial_print_debug('1');
	       udp_ntp_receive(ntohs(udpPkt->len));
	       break;
#endif
#ifdef BOOTP
	    case IPPORT_BOOTPC:
	       serial_print_debug('2');
	       bootp_receive();
	       break;
#endif
	    default:
	       serial_print_debug('?');
	       break;
	 }
      }
      else
      {
	 serial_print_string("\r\nUDP checksum error\r\n");
#ifdef DEBUG
	 cs8900_dump(rxBuffer, BUFFERLEN);
#endif
      }
   }
   else
   {
      serial_print_string("\r\nUDP length mismatch\r\n");
   }
}

